#pragma once

#pragma pack(push, 4)

//
// a structure defining a SMPTE time code frame
//

typedef struct _SMPTE
{
	unsigned char           fu;         // frame units
	unsigned char           u1;         // user group 1
	unsigned char           ft;         // frame tens
	unsigned char           u2;         // user group 2
	unsigned char           su;         // second units
	unsigned char           u3;         // user group 3
	unsigned char           st;         // second tens
	unsigned char           u4;         // user group 4
	unsigned char           mu;         // minute units
	unsigned char           u5;         // user group 5
	unsigned char           mt;         // minute tens
	unsigned char           u6;         // user group 6
	unsigned char           hu;         // hour units
	unsigned char           u7;         // user group 7
	unsigned char           ht;         // hour tens
	unsigned char           u8;         // user group 8
} SMPTE, *PSMPTE;


//
// a structure defining an IRIG-B time code frame
//

typedef struct _IRIG
{
	unsigned char           su;         // second units
	unsigned char           st;         // second tens
	unsigned char           mu;         // minute units
	unsigned char           mt;         // minute tens
	unsigned char           hu;         // hour units
	unsigned char           ht;         // hour tens
	unsigned char           du;         // day units
	unsigned char           dt;         // day tens
	unsigned char           dh;         // day hundreds
	unsigned char			cf1;		// control functions 1 (see TCR-500 documentation)
	unsigned char			cf2;		// control functions 2 (see TCR-500 documentation)
	unsigned char			cf3;		// control functions 3 (see TCR-500 documentation)
	unsigned char			cf4;		// control functions 4 (see TCR-500 documentation)
	unsigned char			cf5;		// control functions 5 (see TCR-500 documentation)
	unsigned char			cf6;		// control functions 6 (see TCR-500 documentation)
	unsigned char			cf7;		// control functions 7 (see TCR-500 documentation)
} IRIG, *PIRIG;

// Communication statuses between TCR device driver and TCR1000

#define HW_STATUS_NORMAL        0x00
#define HW_STATUS_CARDCOMMFAIL  0x01

// time code types

#define TC_TYPE_SMPTE			0x00000001L
#define TC_TYPE_IRIG			0x00000002L	// deprecated
#define TC_TYPE_IRIG_B1			0x00000002L	// originally, IRIG was only B1
#define TC_TYPE_IRIG_B			0x00000003L
#define TC_TYPE_UNDEFINED		0x00008000L

// read time code statuses (mapped to kernel-driver statuses)

#define DLL_TC_STATUS_NORMAL	0x00	// time code read was successful	
#define DLL_TC_STATUS_INVALID	0x01	// time code data is invalid
#define DLL_TC_STATUS_LOST		0x02	// time code not available
#define DLL_TC_STATUS_DRVFAIL	0x03	// TCR driver communication failure
#define DLL_TC_STATUS_CALIB		0x04	// time code signal calibration in progress

// GPS-200A statuses and MCR1000/MCR5000 SMPTE with Leitch encoding and Masterclock proprietary control bits

#define GPS_STATUS_FREEWHEEL	0x00000001

// DLL intialization statuses

#define DLL_INIT_OK				0		// initialization was successful
#define DLL_INIT_CFGERR			3		// registry configuration is invalid or missing
#define DLL_INIT_NOKMD			4		// unable to find the driver, which is required
#define DLL_INIT_TCPROBLEM		5		// unable to detect time code type
#define DLL_INIT_INV_DLL		6		// problem loading DLL or its functions
#define DLL_INIT_UNEXPECTED		255		// unexpected failure (contact technical support)

// asynchronous notifications (maps to TCR interrupts)

#define ASYNC_OKAY				0x00000000		//
#define ASYNC_TIMESET			0x00000001		// once/minute interrupt (sec=30, frame=0)
#define ASYNC_TIMEGAIN			0x00000002		// time code regained from lost state interrupt
#define ASYNC_TIMELOSS			0x00000004		// time code lost interrupt
#define ASYNC_ENDTC				0x00000008		// once/frame interrupt (after time code, before sync word)
#define ASYNC_STARTTC			0x00000010		// once/frame interrupt (before time code, after sync word)
#define ASYNC_PERSEC			0x00000020		// once/second interrupt (frame=0)			
#define ASYNC_ALL				0x7fffffff
#define ASYNC_MASK_ERROR		0x80000000

union uTimeCode
{
	IRIG        irig;
	SMPTE       smpte;
};

typedef struct _TCRTIME {

	// What type of timecode are we decoding

	DWORD			dwTimeCodeType;

	// raw time code values
	uTimeCode rawTc;

	// if a time given asychronously (by an interrupt on TCR), contains high-performance counter
	// reading at time of interrupt

	LARGE_INTEGER	intPerfCount;

	// decoded time

	WORD			wHour;
	WORD			wMinute;
	WORD			wSecond;
	WORD			wMilliseconds;
	WORD			wMonth;
	WORD			wDay;
	WORD			wYear;

	// validity flag (does checksum pass?  TCR-500/500PCI only)

	BYTE			valid;

} TCRTIME, *PTCRTIME;

typedef DWORD (__stdcall *PTCRASYNC_CALLBACK_ROUTINE)(DWORD dwStatus, PVOID data, DWORD dataLen); 

#pragma pack(pop)
