#pragma once

//
//	Software Configuration Management class
//

#include <wtypes.h>
#include "tcrcommon.h"

#define MAXTIMEZONES		512

typedef struct _TIME_ZONE_DESC
{
	char						tzAbbrev[64];	// registry name for time zone
	char						tzDesc[256];	// verbose description of time zone	
	REGISTRY_TIMEZONE_INFO		tzInfo;			// time zone information

} TIME_ZONE_DESC;

typedef struct
{
		// Windows NT registry key for our time zone information (if available)
		char								tzRegistryKey[256];

		// verbose description of time zone	
		char								tzDesc[256];
		
		// time zone information
		TIME_ZONE_INFORMATION				tzInfo;

		// TRUE if date encoded in source time code
		DWORD								dateAvailable;

		// TRUE if DST is encoded in source time code
		DWORD								dstEncoded;

		// time code product/source	
		DWORD								timeCodeSource;
			
		// TRUE if clock to be set during time code freewheeling
		// GPS-200A/MCR1000/MCR5000 operation statuses when Masterclock, Inc proprietary control
		// bits are included in SMPTE time code, Leitch date encoding)
		DWORD								freewheelSynchronize;

		// TRUE if warning to be displayed during time code freewheeling
		// GPS-200A/MCR1000/MCR5000 operation statuses when Masterclock, Inc proprietary control
		// bits are included in SMPTE time code, Leitch date encoding)
		DWORD								freewheelDisplay;

		// TRUE if pop-up warning to be displayed when time code losses occur
		DWORD								tcLossNotify;
	
		//
		// The following options/flags will be special or undocumented additional 
		// features.
		//

		// TRUE when aggressive timesetting is desired (set time immediate after each loss, etc.)
		//DWORD								setTimeAggressive;

		// TRUE when set to use crude (less restrictive) time-setting algorithm)
		//DWORD								useCrudeTimeSet;

} REGISTRY_KEYS;

class CSoftwareConfig
{
	public:
		CSoftwareConfig();
		
		REGISTRY_KEYS  m_srKeys;

		TIME_ZONE_DESC m_TimeZones[MAXTIMEZONES];
		UINT16         m_NumTimeZones;

		UINT32  SaveConfigFromAPI(TCR1000_CONFIGURATION *pApiSetConfig);
		UINT32  CheckConfigFromAPI(TCR1000_CONFIGURATION *pApiSetConfig);
		void    GetConfigForAPI(TCR1000_CONFIGURATION *pApiGetConfig);
		BOOL    GetRegistryTimeZones(void);
		BOOL	IsTzDescUTC();
		void	SetTzDescToUTC(REGISTRY_KEYS *psrMyKeys);
		int		LoadConfigFromRegistry();
		int		SaveConfigToRegistry();
		//void	GetImagePath(TCHAR *destBuff, int iBuffSize);

		void	operator  =(CSoftwareConfig &);
		int	    operator ==(CSoftwareConfig &);
		int	    operator !=(CSoftwareConfig &);

	private:
		int	    CompareTimeZoneDesc(TIME_ZONE_INFORMATION *t1, TIME_ZONE_INFORMATION *t2);
		void    GetRegistryPath(TCHAR *pPath, int iPathLength);
		void    GetKeyDefaults(REGISTRY_KEYS *psrMyKeys);
		void	VerifyRegistryKeys();

};
